package org.tekkotsu.ui.editor.actions;

import java.util.HashMap;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.jface.action.Action;
import org.tekkotsu.ui.editor.StateMachineEditor;
import org.tekkotsu.ui.editor.model.AbstractModel;
import org.tekkotsu.ui.editor.model.ModelData;
import org.tekkotsu.ui.editor.model.SourceNodeModel;
import org.tekkotsu.ui.editor.model.SourceTransitionModel;
import org.tekkotsu.ui.editor.model.LayoutData;
import org.tekkotsu.ui.util.Debugger;

public class CreateTransitionAction extends Action {
	private StateMachineEditor editor;
	private SourceTransitionModel model;
	private CreateRequest request;

	public CreateTransitionAction(StateMachineEditor editor, SourceTransitionModel model) {
		super(model.getId(), AS_CHECK_BOX);
		this.editor = editor;
		this.model = model;
		request = new CreateRequest();
	}

	public SourceTransitionModel getModel() {
		return model;
	}

	public CreateRequest getRequest() {
		return request;
	}

	/**
	 * generate source/target node for single pair transition model
	 */
	private void createSourceTarget() {
		ModelData src = editor.getLayoutData().getModelData();
		
		List<String> sourceNodeList = model.getSourceNodes();
		List<String> destNodeList   = model.getDestNodes();
		
		//single pair transition model
		assert(sourceNodeList.size() == 1);
		assert(destNodeList.size() == 1);
		
		String sourceNodeID = sourceNodeList.get(0);
		String destNodeID   = destNodeList.get(0);
		
		SourceNodeModel sourceNode = src.getNode(sourceNodeID);
		SourceNodeModel destNode   = src.getNode(destNodeID);
		
		CreateNodeAction createSrc = new CreateNodeAction(editor,sourceNode);
		CreateNodeAction createDest = new CreateNodeAction(editor,destNode);
		createSrc.getRequest().setLocation(request.getLocation());
		createDest.getRequest().setLocation(
				(new Point(request.getLocation())).translate(40, 40));
		createSrc.run();
		createDest.run();
		HashMap<String, AbstractModel> data = new HashMap<String, AbstractModel>();
		data.put("_source", editor.getLayoutData().getPartChild(model.getSourceNodes().get(0))
			);
		data.put("_target", editor.getLayoutData().getPartChild(model.getDestNodes().get(0))
			);
		request.setExtendedData(data);
	}

	@Override
	public void run() {
		EditDomain domain = editor.getEditDomain();
		CommandStack stack = domain.getCommandStack();
		GraphicalViewer view = editor.getGraphicalViewer();
		request.setFactory(model.getFactory((LayoutData) view.getContents()
				.getModel()));
		Debugger.printDebug(Debugger.DEBUG_ALL,
				"Create transition action runned: "
						+ request.getNewObjectType());
		if (!model.isMultiTransition()) {
			createSourceTarget();
		}
		stack.execute(view.getContents().getEditPolicy(
				EditPolicy.LAYOUT_ROLE).getCommand(request));
		editor.setDirty();
	}
}
