package org.tekkotsu.ui.storyboard.actions;

import org.eclipse.core.runtime.Path;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.tekkotsu.ui.editor.StateMachineEditor;
import org.tekkotsu.ui.rcp.StoryboardView;
import org.tekkotsu.ui.storyboard.views.ContentProvider;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;
import org.tekkotsu.ui.util.Debugger;
import org.tekkotsu.ui.util.FilesystemUtil;

public class LoadTraceAction extends Action {
	
	private String filepath;
	private StoryboardViewer viewer;
	private StateMachineEditor editor;
	private ContentProvider content;
	
	public LoadTraceAction()
	{
		this(null);
	}
	
	/**
	 * 
	 * @param filepath if null prompt user for path
	 * @param editore 
	 */
	public LoadTraceAction(String filepath)
	{
		super("Load Trace");
		this.filepath = filepath;
		
		this.viewer = StoryboardView.view.viewer;
		this.editor = StoryboardView.view.editor;
		this.content = StoryboardView.view.content;
	}
	
	private String queryFile() {
		FileDialog dialog = new FileDialog(viewer.getControl()
				.getShell(), SWT.OPEN);
		dialog.setFilterExtensions(new String[] { "*.tse" });
		dialog.setText("Open Trace File"); //$NON-NLS-1$

		FilesystemUtil.setFileDialogStartPath(dialog);
		String path = dialog.open();
		FilesystemUtil.stickyPathSet(path);

		if (path != null && path.length() > 0)
			return path;
		return null;
	}

	@Override
	public void run() {
		ContentProvider oldContent = content;
		try {
			if( filepath == null )
			{
				filepath = queryFile();
				
				if (filepath == null) {
					// cancel button was hit
					return;
				}
			}
			
			content.parseNewTrace(new Path(filepath));
			content.endBlocks(viewer.getTimeLine().getTimeCursor());
			viewer.setContent(content);
			viewer.resetInput(editor.getLayoutData());
		} catch (Exception e) {
			Debugger.printThrowable(e);
			String msg = "Cannot open trace file " + filepath;
			MessageDialog.openError(viewer.getControl().getShell(),
					"Problem", msg);
			content = oldContent;
			viewer.setContent(content);
		}
		viewer.updateAllInput(editor.getLayoutData());
	}
}