/*
 * Created on Nov 17, 2004
 */
package org.tekkotsu.ui.storyboard.objects;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.tekkotsu.ui.editor.model.MultiTransitionModel;
import org.tekkotsu.ui.editor.model.SourceTransitionModel;
import org.tekkotsu.ui.editor.model.StateNodeModel;
import org.tekkotsu.ui.editor.model.LayoutData;
import org.tekkotsu.ui.storyboard.model.TransitionModel;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;
import org.tekkotsu.ui.util.ColorGenerator;

/**
 * @author asangpet
 *
 */
public class TransitionViewObject extends AbstractViewObject {
		
	TransitionModel model;
	MultiTransitionModel trans;
	List<StateNodeModel> nodeList;
	
	public TransitionViewObject(TransitionModel model, MultiTransitionModel trans, StoryboardViewer viewer) {
		super(viewer);
		setModel(model);
		setColor(new RGB(0,0,0));
		this.trans = trans;
		
		LayoutData view = (LayoutData)trans.getParent();
		SourceTransitionModel strans = (SourceTransitionModel)trans.getSource();
		int ymin=Integer.MAX_VALUE;
		int ymax=Integer.MIN_VALUE;
				
		List<String> nodes = new ArrayList<String>();
		nodes.addAll(strans.getSourceNodes());				
		nodes.addAll(strans.getDestNodes());
		nodeList = new ArrayList<StateNodeModel>();
		
		for( String node : nodes ) {
			StateNodeModel state = (StateNodeModel)view.getPartChild(node);
			nodeList.add(state);
			int top = state.getConstraint().y;
			int bottom = state.getConstraint().y+state.getConstraint().height;
			if (top<ymin) ymin=top;
			if (bottom>ymax) ymax=bottom;
		}
		
		int xoff = timeline.getOffsetOfTime(model.getTime());
		
		setConstraint(xoff-3,ymin,7,ymax-ymin);
		setColor(trans.getColor().getRGB());
		this.setToolTip(new Label(trans.getId()));
	}
	
	public void setColor(RGB c) {
		colReg.put(COLOR_BASE,c);
		colReg.put(COLOR_FADE, ColorGenerator.fade(c,100));
		updateColors();
	}
	
	@Override
	public void updateColors()
	{
		setBackgroundColor(colReg.get(COLOR_BASE));
		if(trans != null)
		{
			setForegroundColor(trans.getColor());
		}
	}
	
	@Override
	public void setBackgroundColor( Color c )
	{
		if( c == null ) c = colReg.get(COLOR_BORDER);
		super.setBackgroundColor(c);
	}
	
	@Override
	public void setForegroundColor( Color c )
	{
		if( c == null ) c = colReg.get(COLOR_BORDER);
		super.setForegroundColor(c);
	}
	
	@Override
	public void paint(Graphics g2d) {
		Rectangle bound = this.getConstraint();
		Rectangle clArea = this.getClientArea();
		
		g2d.setBackgroundColor(getLocalBackgroundColor());	
		g2d.setForegroundColor(getLocalForegroundColor());
		
		int[] xp = new int[] {clArea.x,clArea.x+3,clArea.x+6,clArea.x+3};
		
		int miny=Integer.MAX_VALUE, maxy=-1;
		if (nodeList!=null) {
			Iterator<StateNodeModel> iter = nodeList.iterator();			
			while (iter.hasNext()) {
				StateNodeModel state = iter.next();
				Rectangle rect = state.getConstraint();
				int yoff = clArea.y+(rect.y-bound.y)+rect.height/2;
				if (yoff<miny) miny = yoff;
				if (yoff>maxy) maxy = yoff;
				int[] yp = new int[] {yoff,yoff-3,yoff,yoff+3};
				PointList plist = new PointList(4);
				plist.addPoint(xp[0],yp[0]);
				plist.addPoint(xp[1],yp[1]);
				plist.addPoint(xp[2],yp[2]);
				plist.addPoint(xp[3],yp[3]);				
				g2d.fillPolygon(plist);
			}		
		}
		if (miny==Integer.MAX_VALUE) miny=clArea.y;
		if (maxy==-1) maxy=clArea.y+bound.height;
		
		if(getActive())
		{
			g2d.setLineWidth(6);
		}
		g2d.drawLine(3+clArea.x,miny,3+clArea.x,maxy);
		g2d.setLineWidth(1);
	}
	/**
	 * @return Returns the model.
	 */
	@Override
	public TransitionModel getModel() {
		return model;
	}

	private void setModel(TransitionModel model) {
		this.model = model;
	}

	@Override
	public String getID() {
		return model.getID();
	}
	
}
