/*
 * Created on Nov 7, 2004
 */
package org.tekkotsu.ui.storyboard.views;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.tekkotsu.ui.storyboard.ResourceRegistry;
import org.tekkotsu.ui.storyboard.model.ImageModel;
import org.tekkotsu.ui.util.ImageDialog;

/**
 * @author asangpet
 * 
 */
public class ImageViewer extends Viewer {
	Canvas canvas;
	Image image;
	ImageModel imageModel;

	public ImageViewer(Composite parent) {
		final Composite _parent = parent;
		final ImageViewer _this = this;

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		parent.setLayout(layout);

		canvas = new Canvas(parent, SWT.FILL);
		canvas.setSize(200, 200);
		canvas.addPaintListener(new PaintListener() {
			public void paintControl(PaintEvent e) {
				if (image != null) {
					ImageData imageData = image.getImageData();
					canvas.setSize(imageData.width, imageData.height);
					e.gc.drawImage(image, 0, 0);
				} else
					e.gc.fillRectangle(0, 0, _parent.getSize().x, _parent
							.getSize().y);
			}
		});
		canvas.addMouseListener(new MouseAdapter(){
			@Override
			public void mouseUp (MouseEvent e) {
				ImageData data = _this.imageModel.getImageData();
                Shell shell = ResourceRegistry.getInstance().getDisplay()
                        .getActiveShell();
                Device device = ResourceRegistry.getInstance().getDisplay();
                ImageDialog imgDialog = new ImageDialog(shell, new Image(
                        device, data));
                imgDialog.setTitle("Image: "
                        + _this.imageModel.getImageName());
                imgDialog.open();
			}
		});
	}	

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IInputProvider#getInput()
	 */
	@Override
	public Object getInput() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.viewers.Viewer#setSelection(org.eclipse.jface.viewers
	 * .ISelection, boolean)
	 */
	@Override
	public void setSelection(ISelection selection, boolean reveal) {
		// nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	@Override
	public ISelection getSelection() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	@Override
	public void refresh() {		
		if (image != null)
			image.dispose();
		
		if(imageModel != null)
		{
			ImageData imageData = imageModel.getImageData();

			int w = canvas.getParent().getSize().x;
			int h = canvas.getParent().getSize().y;
			if (w > imageData.width)
				w = imageData.width;
			if (h > imageData.height)
				h = imageData.height;
			imageData = imageData.scaledTo(w, h);
			image = new Image(getControl().getDisplay(), imageData);
		}
		
		canvas.redraw();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#setInput(java.lang.Object)
	 */
	@Override
	public void setInput(Object input) {
		// nothing
	}

	public ImageModel getImageModel() {
		return imageModel;
	}

	public void setModel(ImageModel model) {		
		imageModel = model;
		refresh();
	}

	@Override
	public Control getControl() {
		return canvas.getShell();
	}
}
