/*
 * Created on Nov 7, 2004
 */
package org.tekkotsu.ui.storyboard.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.core.runtime.Path;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.Panel;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;
import org.tekkotsu.ui.editor.StateMachineEditor;
import org.tekkotsu.ui.editor.model.MultiTransitionModel;
import org.tekkotsu.ui.editor.model.StateNodeModel;
import org.tekkotsu.ui.editor.model.LayoutData;
import org.tekkotsu.ui.editor.netcode.RobotConnector;
import org.tekkotsu.ui.rcp.StoryboardView;
import org.tekkotsu.ui.storyboard.model.AbstractModel;
import org.tekkotsu.ui.storyboard.model.BlockModel;
import org.tekkotsu.ui.storyboard.model.ImageModel;
import org.tekkotsu.ui.storyboard.model.LogModel;
import org.tekkotsu.ui.storyboard.model.TransitionModel;
import org.tekkotsu.ui.storyboard.objects.AbstractViewObject;
import org.tekkotsu.ui.storyboard.objects.BlockViewObject;
import org.tekkotsu.ui.storyboard.objects.ImageViewObject;
import org.tekkotsu.ui.storyboard.objects.LogViewObject;
import org.tekkotsu.ui.storyboard.objects.TransitionViewObject;
import org.tekkotsu.ui.storyboard.panels.AbstractTimePanel;
import org.tekkotsu.ui.storyboard.panels.LogBar;
import org.tekkotsu.ui.storyboard.panels.Storyboard;
import org.tekkotsu.ui.storyboard.panels.TimeLine;
import org.tekkotsu.ui.util.Debugger;

/**
 * @author asangpet
 * 
 */
public class StoryboardViewer extends Viewer implements PropertyChangeListener {

	public static String startupTraceFile;
	public boolean traceRunning = false;

	// Other relevant views
	ImageViewer imageViewer;

	// Supplies the display?
	FigureCanvas canvas;

	// The panel containing the timeline/logBar/storyboard
	Panel panel;
	public TimeLine timeline;
	LogBar logbar;
	Storyboard storyboard;

	public LayoutData viewModel;

	// Parses the XML data
	ContentProvider content;

	ISelection curSelect;

	XYLayout contentsLayout = new XYLayout();

	public StoryboardViewer(Composite parent, StoryboardView hostView,
			StateMachineEditor editor, ContentProvider content,
			ImageViewer imageViewer) {
		try {
			final StoryboardViewer _this = this;
			this.content = content;

			final StoryboardView storyboardView = hostView;

			GridData gd;
			GridLayout layout;

			gd = new GridData();
			layout = new GridLayout();
			parent.setLayout(layout);
			layout.marginHeight = 0;
			layout.marginWidth = 0;
			layout.numColumns = 1;
			layout.verticalSpacing = 0;
			layout.horizontalSpacing = 0;

			/*
			 * GUI Controls
			 */
			gd = new GridData(SWT.FILL, SWT.TOP, true, false);
			gd.horizontalSpan = 1;

			Composite allControls = new Composite(parent, SWT.NORMAL);
			layout = new GridLayout();
			allControls.setLayout(layout);
			allControls.setLayoutData(gd);
			layout.marginHeight = 0;
			layout.numColumns = 1;
			layout.verticalSpacing = 0;
			layout.horizontalSpacing = 0;

			gd = new GridData(SWT.FILL, SWT.FILL, true, false);
			gd.horizontalSpan = 1;

			Composite controlComposite = new Composite(allControls, SWT.NORMAL);
			controlComposite.setLayoutData(gd);
			layout = new GridLayout();
			controlComposite.setLayout(layout);
			layout.numColumns = 5;
			layout.verticalSpacing = 0;
			layout.marginHeight = 0;

			final Button recordButton = new Button(controlComposite, SWT.NONE);
			final Button pauseButton = new Button(controlComposite, SWT.NONE);
			final Button saveButton = new Button(controlComposite, SWT.NONE);
			final Button loadButton = new Button(controlComposite, SWT.NONE);
			final Button redrawButton = new Button(controlComposite, SWT.NONE);

			gd = new GridData(GridData.FILL_HORIZONTAL);
			gd.horizontalSpan = 1;
			recordButton.setLayoutData(gd);
			pauseButton.setLayoutData(gd);
			saveButton.setLayoutData(gd);
			loadButton.setLayoutData(gd);
			redrawButton.setLayoutData(gd);

			recordButton.setText("Record New Trace");
			pauseButton.setText("Pause Trace");
			saveButton.setText("Save Trace As");
			loadButton.setText("Load Trace");
			redrawButton.setText("Redraw Trace");

			recordButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent event) {					
					StateMachineEditor editor = (StateMachineEditor) PlatformUI
							.getWorkbench().getActiveWorkbenchWindow()
							.getActivePage().getActiveEditor();
					editor.robotConnector.props.firePropertyChange(
							RobotConnector.EVENT_TRACE_CLEAR, null, null);
					editor.robotConnector.props.firePropertyChange(
							RobotConnector.EVENT_TRACE_MODE_ENTER, null, null);
					editor.robotConnector.start();
					recordButton.setText("Recording Trace...");
					recordButton.setEnabled(false);
					pauseButton.setText("Pause Trace");
					pauseButton.setEnabled(true);
					loadButton.setEnabled(false);
					_this.reset();
					traceRunning = true;
				}
			});

			pauseButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					StateMachineEditor editor = (StateMachineEditor) PlatformUI
							.getWorkbench().getActiveWorkbenchWindow()
							.getActivePage().getActiveEditor();
					RobotConnector modelConnector = editor.robotConnector;
					if (traceRunning) {
						modelConnector.props.firePropertyChange(
								RobotConnector.EVENT_TRACE_MODE_EXIT, null,
								null);
						modelConnector.stop();
						pauseButton.setText("Resume Trace");
						recordButton.setText("Record New Trace");
						recordButton.setEnabled(true);
						loadButton.setEnabled(true);
						traceRunning = false;
					} else {
						modelConnector.props.firePropertyChange(
								RobotConnector.EVENT_TRACE_MODE_ENTER, null,
								null);
						modelConnector.resume();
						pauseButton.setText("Pause Trace");
						recordButton.setText("Recording Trace...");
						recordButton.setEnabled(false);
						loadButton.setEnabled(false);
						traceRunning = true;
					}
				}
			});
			pauseButton.setEnabled(false);

			saveButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					Action act = storyboardView.makeSaveTraceAction();
					act.run();
				}
			});

			loadButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					Action act = storyboardView.makeLoadTraceAction();
					act.run();
				}
			});

			redrawButton.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent event) {
					setContent(_this.content);
					updateAllInput(_this.viewModel);
				}
			});

			/*
			 * Main Display
			 */
			gd = new GridData(SWT.FILL, SWT.FILL, true, true);
			gd.horizontalSpan = 1;

			canvas = new FigureCanvas(parent);
			canvas.setLayoutData(gd);

			panel = new Panel();
			panel.setBackgroundColor(ColorConstants.white);
			canvas.setContents(panel);

			panel.setLayoutManager(contentsLayout);

			timeline = new TimeLine(panel, this);
			logbar = new LogBar(panel, this, timeline.getMarker());
			storyboard = new Storyboard(panel, this, timeline.getMarker());

			/*
			 * Connect everything together
			 */
			timeline.addPropertyChangeListener(storyboard);
			timeline.addPropertyChangeListener(logbar);

			logbar.addPropertyChangeListener(storyboard);
			logbar.addPropertyChangeListener(timeline);

			storyboard.addPropertyChangeListener(timeline);
			storyboard.addPropertyChangeListener(logbar);
		} catch (Exception e) {
			Debugger.printThrowable(e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#getControl()
	 */
	@Override
	public Control getControl() {
		return canvas;
	}

	public Rectangle getVisibleViewport() {
		Viewport viewport = getCanvas().getViewport();
		Point viewLoc = viewport.getViewLocation().getCopy();
		Rectangle viewRect = viewport.getBounds().getCopy();
		viewRect.setLocation(viewLoc);

		return viewRect;
	}
	
	public void pushVisibleViewportRight(final int x) {
		if( traceRunning ) {
			Debugger.printDebug("PUSH VIEWPORT RIGHT");
			getControl().getDisplay().syncExec(new Runnable() {
				public void run() {
					final Viewport viewport = getCanvas().getViewport();
					final Rectangle OLD = getVisibleViewport();
					viewport.setHorizontalLocation(x-OLD.width);
				}
			});
		}
	}

	public FigureCanvas getCanvas() {
		return canvas;
	}

	public ImageViewer getImageViewer() {
		return imageViewer;
	}

	public void setImageViewer(ImageViewer imageViewer) {
		this.imageViewer = imageViewer;
	}

	public void setContent(ContentProvider content) {
		this.content = content;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IInputProvider#getInput()
	 */
	@Override
	public Object getInput() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
	 */
	@Override
	public ISelection getSelection() {
		Debugger.printDebug(Debugger.DEBUG_ALL, "getting selection");
		return curSelect;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	@Override
	public void refresh() {
		RuntimeView.getInstance().clear();
		storyboard.timeCursorUpdated();
		timeline.timeCursorUpdated();
		logbar.timeCursorUpdated();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.Viewer#setInput(java.lang.Object)
	 */
	@Override
	public void setInput(Object input) {
		resetInput((LayoutData) input);
	}

	public TimeLine getTimeLine() {
		return timeline;
	}

	private void update(final AbstractModel model) {
		Debugger.printDebug("[StoryboardViewer] update(" + model + ")");
		final Display display = canvas.getDisplay();
		final StoryboardViewer _this = this;

		final int MAX_TIME = Math.max(model.getStart(), model.getEnd());
		timeline.ensureTimeLimit(MAX_TIME);
		logbar.ensureTimeLimit(MAX_TIME);
		storyboard.ensureTimeLimit(MAX_TIME);

		display.syncExec(new Runnable() {
			public void run() {
				AbstractViewObject child = null;
				AbstractTimePanel parent = storyboard;

				if (model instanceof ImageModel) {
					ImageModel obj = (ImageModel) model;
					StateNodeModel state = (StateNodeModel) viewModel
							.getPartChild(obj.getID());
					// user in-state image object
					child = new ImageViewObject(obj, state, _this);
					parent = storyboard;
				} else if (model instanceof LogModel) {
					LogModel obj = (LogModel) model;
					StateNodeModel state = (StateNodeModel) viewModel
							.getPartChild(obj.getID());
					if (state != null) {
						// user in-state log object
						child = new LogViewObject(obj, state, _this);
					} else {
						// system log object
						child = new LogViewObject(obj, null, _this);
					}
					parent = logbar;
				} else if (model instanceof TransitionModel) {
					TransitionModel obj = (TransitionModel) model;
					Object editor_obj = viewModel.getPartChild(obj.getID());
					
					if(editor_obj == null) {
						Debugger.printError("Null trans during trace");
						return;
					}
					
					if (editor_obj instanceof MultiTransitionModel) {
						MultiTransitionModel trans = (MultiTransitionModel) editor_obj;						
						child = new TransitionViewObject(obj, trans, _this);
					}
					parent = storyboard;
				} else if (model instanceof BlockModel) {
					BlockModel obj = (BlockModel) model;
					StateNodeModel state = (StateNodeModel) viewModel
							.getPartChild(obj.getID());
					if(state == null) {
						Debugger.printError("Null state during trace");
						return;
					}
					child = new BlockViewObject(_this, obj, state);
					parent = storyboard;
				}

				if (child != null) {
					parent.update(child);
				}
			}
		});
	}

	public void resetInput(LayoutData input) {
		reset();
		updateAllInput(input);
	}

	public void updateInput(LayoutData input) {
		viewModel = input;

		for (AbstractModel obj : content.getUpdate()) {
			update(obj);
		}

		refresh();
	}

	public void updateAllInput(LayoutData input) {
		viewModel = input;
		
		for (AbstractModel obj : content.getElements()) {
			update(obj);
		}
		
		try {
			final Rectangle OLD = storyboard.getConstraint();
			if( OLD != null && viewModel != null ) {
				storyboard.setConstraint(OLD.x, OLD.y, OLD.width, viewModel.getNeededHeight());
			}
		} catch (Exception e) {
			Debugger.printThrowable(e);
		}
		
		refresh();
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		final PropertyChangeEvent _event = evt;
		if (evt.getPropertyName().equals(RobotConnector.EVENT_DATA_UPDATE)) {
			//ignore
		} else if (evt.getPropertyName().equals(RobotConnector.EVENT_TRACE_UPDATE_TIME)) {
			Integer time = (Integer)evt.getNewValue();
			final int TIME = time.intValue();
			getControl().getDisplay().syncExec(new Runnable() {
				public void run() {
					storyboard.setTraceTime(TIME);
					logbar.setTraceTime(TIME);
					timeline.setTraceTime(TIME);
				}
			});
		} else if (evt.getPropertyName().equals(RobotConnector.EVENT_TRACE_UPDATE)) {
			Debugger.printStoryboardEvent("PROP CHANGE * P_TRACE_UPDATE");
			Debugger.printStoryboardEvent((String) evt.getNewValue());

			getControl().getDisplay().syncExec(new Runnable() {
				public void run() {
					// update trace
					content.getUpdate((String) _event.getNewValue());
					setContent(content);
					updateInput(StoryboardView.view.editor.getLayoutData());
				}
			});
		} else if (evt.getPropertyName().equals(
				RobotConnector.EVENT_TRACE_MODE_ENTER)) {
			Debugger.printStoryboardEvent("PROP CHANGE * P_TRACE_MODE_ENTER");
		} else if (evt.getPropertyName().equals(
				RobotConnector.EVENT_TRACE_MODE_EXIT)) {
			Debugger.printStoryboardEvent("PROP CHANGE * P_TRACE_MODE_EXIT");

			getControl().getDisplay().syncExec(new Runnable() {
				public void run() {
					content.endBlocks(timeline.getTimeCursor());
				}
			});
		} else if (evt.getPropertyName().equals(
				RobotConnector.EVENT_TRACE_CLEAR)) {
			Debugger.printStoryboardEvent("PROP CHANGE * P_TRACE_CLEAR");

			getControl().getDisplay().syncExec(new Runnable() {
				public void run() {
					// clear trace
					content.clear();
					setContent(content);
					resetInput(StoryboardView.view.editor.getLayoutData());
				}
			});
		} else if (evt.getPropertyName().equals(LayoutData.P_EXECUTION_TRACE)) {
			Debugger.printStoryboardEvent("PROP CHANGE * P_EXECUTION_TRACE");
			// load trace from file
			ContentProvider oldContent = content;
			try {
				String contentname = (String) evt.getNewValue();
				content.parseNewTrace(new Path(contentname));
				setContent(content);
				resetInput(StoryboardView.view.editor.getLayoutData());
			} catch (Exception e) {
				Debugger.printThrowable(e);
				content = oldContent;
				setContent(content);
				resetInput(StoryboardView.view.editor.getLayoutData());
			}
		} else if (evt.getPropertyName().equals(StateMachineEditor.DIRTY)) {
			resetInput(viewModel);
		}
	}

	@Override
	public void setSelection(ISelection selection, boolean reveal) {
		// nothing
	}

	public void reset() {
		timeline.reset();
		logbar.reset();
		storyboard.reset();
		refresh();
	}
}
